c ParaDyn - Parallel DYNAMO - molecular dynamics with EAM potentials
c 
c Authored by Steve Plimpton
c   (505) 845-7873, sjplimp@cs.sandia.gov
c   Dept 9221, MS 1111, Sandia National Labs, Albuquerque, NM  87185-1111
c
c Based on the serial DYNAMO code authored by
c   Stephen Foiles (foiles@ca.sandia.gov), Sandia National Labs, Livermore, CA
c   Murray Daw (daw@hubcap.clemson.edu), Clemson University
c
c See the README file for more information
c

c user-specified diagnostic routine
c  never called at if idiag = 0 (no input script command)
c  always called with unscaled coordinates
c  called before minimizer/integrator with m = 0
c  called after minimizer/integrator with m = -1
c  called during intermediate times with (a) integrator: m = timestep
c                                        (b) minimizer: m = # of calls to force

c dummy diagnostic routine
c just uncomment this one and delete others
c  if no special diagnostics are desired

c      subroutine diagnostic(m)

c      return
c      end


c Measure of grain boundary diffusion coefficient
c  does 2 types (1 and everything else) if they exist
c  tags which atoms are in boundary once and for all at the beginning
      
      subroutine diagnostic(m)
#include "paradyn.h"
#include "mpif.h"

      external length
      real*8 diffwork(3),diff1(3),diff2(3)
      integer diffcount1,diffcount2
      common /common_d1/ diffwork,diff1,diff2,diffcount1,diffcount2

#ifdef DYNAMIC
      real*8 difforig(3,1)
      integer difftag(1)
      pointer (difforig_ptr,difforig),(difftag_ptr,difftag)
      common /common_d2/ difforig_ptr
      common /common_d3/ difftag_ptr
      data difforig_ptr,difftag_ptr /2*0/
#else
      real*8 difforig(3,maxlocal)
      integer difftag(maxlocal)
      common /common_d2/ difforig
      common /common_d3/ difftag
#endif

 900  format(i8,4f10.4)
      
      if (m.eq.0) then
        
#ifdef DYNAMIC
        call memfree(difforig_ptr)
        call memfree(difftag_ptr)
        call memalloc(difforig_ptr,3*maxlocal,1)
        call memalloc(difftag_ptr,maxlocal,0)
#endif

        diffcount1 = 0
        diffcount2 = 0
        do i = 1,nlocal
          difforig(1,i) = x(1,i)
          difforig(2,i) = x(2,i)
          difforig(3,i) = x(3,i)
          difftag(i) = 0
          if (x(1,i).ge.diag1.and.x(1,i).le.diag2) then
            difftag(i) = itype(i)
            if (difftag(i).eq.1) then
              diffcount1 = diffcount1 + 1
            else
              diffcount2 = diffcount2 + 1
            endif
          endif
        enddo
        
        call mpi_allreduce(diffcount1,itmp,1,mpi_integer,mpi_sum,
     $       mpi_comm_world,ierror)
        diffcount1 = itmp
        call mpi_allreduce(diffcount2,itmp,1,mpi_integer,mpi_sum,
     $       mpi_comm_world,ierror)
        diffcount2 = itmp

        if (diffcount1.eq.0) call error('No diffusing atoms')

        if (diffcount2.eq.0) then
          if (node.eq.0) open (unit=8,file=diagfile)
          if (node.eq.0) write (8,*) 'GB Diffusion',diffcount1
          if (node.eq.0) write (8,900) 0,0.0,0.0,0.0,0.0
        else
          if (node.eq.0) open (unit=8,
     $         file=diagfile(1:length(diagfile))//'.1')
          if (node.eq.0) write (8,*) 'GB Diffusion 1',diffcount1
          if (node.eq.0) write (8,900) 0,0.0,0.0,0.0,0.0
          if (node.eq.0) open (unit=7,
     $         file=diagfile(1:length(diagfile))//'.2')
          if (node.eq.0) write (7,*) 'GB Diffusion 2',diffcount2
          if (node.eq.0) write (8,900) 0,0.0,0.0,0.0,0.0
        endif

      else if (m.eq.-1) then
        
        if (node.eq.0) close (8)
        if (node.eq.0.and.diffcount2.gt.0) close (7)
        
      else
        
        diff1(1) = 0.0D0
        diff1(2) = 0.0D0
        diff1(3) = 0.0D0
        diff2(1) = 0.0D0
        diff2(2) = 0.0D0
        diff2(3) = 0.0D0
        do i = 1,nlocal
          if (difftag(i).ne.0) then
            dx = x(1,i) - difforig(1,i) + perlen(1)*parity(1,i)
            dy = x(2,i) - difforig(2,i) + perlen(2)*parity(2,i)
            dz = x(3,i) - difforig(3,i) + perlen(3)*parity(3,i)
            if (difftag(i).eq.1) then
              diff1(1) = diff1(1) + dx*dx
              diff1(2) = diff1(2) + dy*dy
              diff1(3) = diff1(3) + dz*dz
            else
              diff2(1) = diff2(1) + dx*dx
              diff2(2) = diff2(2) + dy*dy
              diff2(3) = diff2(3) + dz*dz
            endif
          endif
        enddo
        
        call mpi_allreduce(diff1,diffwork,3,mpi_double_precision,
     $       mpi_sum,mpi_comm_world,ierror)
        diff1(1) = diffwork(1)/diffcount1
        diff1(2) = diffwork(2)/diffcount1
        diff1(3) = diffwork(3)/diffcount1
        sum = diff1(1)+diff1(2)+diff1(3)
        if (node.eq.0) write (8,900) m,sum,diff1(1),diff1(2),diff1(3)

        if (diffcount2.gt.0) then
          call mpi_allreduce(diff2,diffwork,3,mpi_double_precision,
     $         mpi_sum,mpi_comm_world,ierror)
          diff2(1) = diffwork(1)/diffcount2
          diff2(2) = diffwork(2)/diffcount2
          diff2(3) = diffwork(3)/diffcount2
          sum = diff2(1)+diff2(2)+diff2(3)
          if (node.eq.0) write (7,900) m,sum,diff2(1),diff2(2),diff2(3)
        endif
        
      endif
      
      return
      end
